package ink.wgink.gateway.handler.route.request;

import ink.wgink.gateway.dao.route.request.IRequestHeaderDao;
import ink.wgink.gateway.exception.SearchException;
import ink.wgink.gateway.handler.BaseHandler;
import ink.wgink.gateway.pojo.result.SuccessResult;
import ink.wgink.gateway.pojo.route.request.RequestHeader;
import ink.wgink.gateway.pojo.route.request.RequestParameter;
import ink.wgink.gateway.pojo.routetype.RouteType;
import ink.wgink.gateway.util.RequestFieldCheckUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.server.ServerRequest;
import org.springframework.web.reactive.function.server.ServerResponse;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.Arrays;
import java.util.List;
import java.util.Optional;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: RequestHeaderHandler
 * @Description: 请求头处理
 * @Author: WangGeng
 * @Date: 2021/5/30 17:00
 * @Version: 1.0
 **/
@Service
public class RequestHeaderHandler extends BaseHandler {

    private IRequestHeaderDao requestHeaderDao;

    public RequestHeaderHandler(IRequestHeaderDao requestHeaderDao) {
        this.requestHeaderDao = requestHeaderDao;
    }

    public Mono<ServerResponse> save(ServerRequest serverRequest) {
        Mono<RequestHeader> requestHeaderMono = serverRequest.bodyToMono(RequestHeader.class);
        return requestHeaderMono.flatMap(requestHeader -> {
            RequestFieldCheckUtil.check(requestHeader);
            setSave(requestHeader);

            RouteType routeTypeExample = new RouteType();
            routeTypeExample.setTitle(requestHeader.getName().trim());
            ExampleMatcher exampleMatcher = ExampleMatcher.matching().withMatcher("name", ExampleMatcher.GenericPropertyMatcher::exact).withIgnoreCase("id");
            Example example = Example.of(routeTypeExample, exampleMatcher);
            return requestHeaderDao.findOne(example).flatMap(
                    r -> Mono.error(new SearchException("Header已经存在"))
            ).switchIfEmpty(requestHeaderDao.save(requestHeader));
        }).then(ServerResponse.ok().contentType(MediaType.APPLICATION_JSON_UTF8).body(Flux.just(new SuccessResult()), SuccessResult.class));
    }

    public Mono<ServerResponse> delete(ServerRequest serverRequest) {
        String ids = serverRequest.pathVariable("ids");
        List<String> idList = Arrays.asList(ids.split(","));
        return requestHeaderDao.findAllById(idList).flatMap(requestHeader -> requestHeaderDao.delete(requestHeader)).then(ServerResponse.ok().contentType(MediaType.APPLICATION_JSON_UTF8).body(Mono.just(new SuccessResult()), SuccessResult.class));
    }

    public Mono<ServerResponse> update(ServerRequest serverRequest) {
        String id = serverRequest.pathVariable("id");
        Mono<RequestHeader> requestHeaderMono = serverRequest.bodyToMono(RequestHeader.class);

        // 查询ID是否存在
        return requestHeaderDao.findById(id).flatMap(requestHeader -> {
            setUpdate(requestHeader);
            return requestHeaderMono.flatMap(rh -> {
                RequestFieldCheckUtil.check(rh);
                requestHeader.setRouteId(rh.getRouteId());
                requestHeader.setType(rh.getType());
                requestHeader.setName(rh.getName());
                requestHeader.setValue(rh.getValue());

                RequestHeader requestHeaderExample = new RequestHeader();
                requestHeaderExample.setRouteId(rh.getRouteId());
                requestHeaderExample.setName(rh.getName());
                ExampleMatcher exampleMatcher = ExampleMatcher.matching()
                        .withMatcher("routeId", ExampleMatcher.GenericPropertyMatcher::exact)
                        .withMatcher("name", ExampleMatcher.GenericPropertyMatcher::exact)
                        .withIgnoreCase("id");
                Example example = Example.of(requestHeaderExample, exampleMatcher);
                // 查询Name是否存在
                return requestHeaderDao.findOne(example).flatMap(erh -> {
                    RequestHeader existRouteHeader = (RequestHeader) erh;
                    // 如果已经存在
                    if (!StringUtils.equals(existRouteHeader.getUuid(), id)) {
                        return Mono.error(new SearchException("参数已经存在"));
                    }
                    return requestHeaderDao.save(requestHeader);
                }).switchIfEmpty(requestHeaderDao.save(requestHeader));
            });
        }).switchIfEmpty(ServerResponse.notFound().build()).then(ServerResponse.ok().contentType(MediaType.APPLICATION_JSON_UTF8).body(Mono.just(new SuccessResult()), SuccessResult.class));
    }

    public Mono<ServerResponse> list(ServerRequest serverRequest) {
        String routeId = serverRequest.pathVariable("routeId");
        Optional<String> keywords = serverRequest.queryParam("keywords");
        Optional<String> type = serverRequest.queryParam("type");

        RequestParameter requestParameter = new RequestParameter();
        requestParameter.setRouteId(routeId);

        ExampleMatcher exampleMatcher = ExampleMatcher.matching()
                .withMatcher("routeId", ExampleMatcher.GenericPropertyMatcher::exact)
                .withIgnoreCase("id");
        if (keywords.isPresent() || type.isPresent()) {
            String keywordTrim = keywords.get().trim();
            String typeTrim = type.get().trim();

            if (StringUtils.isNotBlank(keywordTrim)) {
                requestParameter.setName(keywordTrim);
                exampleMatcher.withMatcher("name", ExampleMatcher.GenericPropertyMatcher::contains);
            }

            if (StringUtils.isNotBlank(typeTrim)) {
                requestParameter.setType(typeTrim);
                exampleMatcher.withMatcher("type", ExampleMatcher.GenericPropertyMatcher::exact);
            }
        }
        Example example = Example.of(requestParameter, exampleMatcher);
        return ServerResponse.ok().contentType(MediaType.APPLICATION_JSON_UTF8).body(requestHeaderDao.findAll(example), RequestHeader.class);
    }

    public Mono<ServerResponse> get(ServerRequest serverRequest) {
        String id = serverRequest.pathVariable("id");
        return requestHeaderDao.findById(id).flatMap(
                requestHeader -> ServerResponse.ok().body(Mono.just(requestHeader), RequestHeader.class)
        ).switchIfEmpty(ServerResponse.notFound().build());
    }

}
